<?php

namespace Teknisa\FIN\Modules\ContaPagar\Factory;

use Doctrine\ORM\EntityManager;
use Teknisa\Libs\Util\DateUtil;
use Teknisa\Model\Financeiro\Entities\Tipoajpagar;
use Teknisa\Model\Financeiro\Entities\Ajbolpag;
use Teknisa\Model\Financeiro\Entities\Baixapaga;
use Teknisa\Model\Financeiro\Entities\Contapagar;
use Teknisa\Model\Financeiro\Entities\Boletpagar;
use Teknisa\Model\Financeiro\Entities\Movbancario;
use Teknisa\Model\Financeiro\Entities\Tipobaixapag;
use Teknisa\Model\Financeiro\Entities\Cheque;
use Teknisa\Model\Contabilidade\Entities\Lanccont;
use Teknisa\Model\Fiscal\Entities\Impbaixarec;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Teknisa\Model\Financeiro\Entities\GeralPag;

/**
 * Class BaixaTituloFactory
 * @package Teknisa\FIN\Modules\ContaPagar\Factory
 */
class BaixaTituloFactory {

    protected $entityManager;

    /**
     * @param EntityManager $entityManager
     */
    public function __construct(EntityManager $entityManager) {
        $this->connection = $entityManager->getConnection();
        $this->entityManager = $entityManager;
    }

    /**
     * Busca Boleto a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @return Boletpagar
     */
    public function buscaBoletPagar($nrorg, $cdfilial, $nrlanbolpag){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolpag'] = $nrlanbolpag;

        /** @var Boletpagar $boletpagar */
        $boletpagar = $this->entityManager->getRepository(Boletpagar::class)->findOneBy($params);

        return $boletpagar;
    }

    /**
     * Busca Boletos a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixapag
     * @return Boletpagar
     */
    public function buscaBoletsPagar($nrorg, $cdfilial, $nrbaixapag){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdfilial']   = $cdfilial;
        $params['nrbaixapag'] = $nrbaixapag;

        /** @var Boletpagar $boletpagar */
        $boletpagar = $this->entityManager->getRepository(Boletpagar::class)->findBy($params);

        return $boletpagar;
    }

    /**
     * Busca Boletos a Pagar
     * @param $nrorg
     * @param $cdfilibaipag
     * @param $nrbaixapag
     * @return Boletpagar
     */
    public function buscaBoletsPagarBaixa($nrorg, $cdfilibaipag, $nrbaixapag){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdfilibaipag'] = $cdfilibaipag;
        $params['nrbaixapag']   = $nrbaixapag;

        /** @var Boletpagar $boletpagar */
        $boletpagar = $this->entityManager->getRepository(Boletpagar::class)->findBy($params);

        return $boletpagar;
    }

    /**
     * Busca Cheque
     * @param $cdreducont
     * @param $nrcheque
     * @return Cheque
     */
    public function buscaCheque($cdreducont, $nrcheque){
        /** @var $params */
        $params = array();
        $params['cdreducont'] = $cdreducont;
        $params['nrcheque']   = $nrcheque;

        /** @var Cheque $empresa */
        $cheque = $this->entityManager->getRepository(Cheque::class)->findOneBy($params);

        return $cheque;
    }

    /**
     * Busca Baixa Paga
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixapag
     * @return Baixapaga
     */
    public function buscaBaixapaga($nrorg, $cdfilial, $nrbaixapag){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdfilial']   = $cdfilial;
        $params['nrbaixapag'] = $nrbaixapag;

        /** @var Baixapaga $baixapaga */
        $baixapaga = $this->entityManager->getRepository(Baixapaga::class)->findOneBy($params);

        return $baixapaga;
    }

    /**
     * Busca Contas a Pagar
     * @param $nrorg
     * @param $cdfilibpag
     * @param $nrlanbolpag
     * @return Contapagar
     */
    public function buscaContaspagarAssoc($nrorg, $cdfilibpag, $nrlanbolpag){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilibpag']  = $cdfilibpag;
        $params['nrlanbolpag'] = $nrlanbolpag;

        /** @var Contapagar $contapagar */
        $contapagar = $this->entityManager->getRepository(Contapagar::class)->findBy($params);

        return $contapagar;
    }

    /**
     * Busca Lancamento Contabil
     * @param $nrorg
     * @param $cdempresa
     * @param $nrlanccont
     * @return Lanccont
     */
    public function buscaLanccont($nrorg, $cdempresa, $nrlanccont){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdempresa']  = $cdempresa;
        $params['nrlanccont'] = $nrlanccont;

        /** @var Lanccont $lanccont */
        $lanccont = $this->entityManager->getRepository(Lanccont::class)->findBy($params);

        return $lanccont;
    }

    /**
     * Busca Movimento Bancário
     * @param $nrorg
     * @param $cdreducont
     * @param $nrmovbanc
     * @return Movbancario
     */
    public function buscaMovbancario($nrorg, $cdreducont, $nrmovbanc){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdreducont'] = $cdreducont;
        $params['nrmovbanc']  = $nrmovbanc;

        /** @var Movbancario $movbancario */
        $movbancario = $this->entityManager->getRepository(Movbancario::class)->findOneBy($params);

        return $movbancario;
    }

    /**
     * Busca Tipo de Baixa Pagar
     * @param $nrorg
     * @param $cdtpbaixapag
     * @return Tipobaixapag
     */
    public function buscaTipobaixapag($nrorg, $cdtpbaixapag){
        /** @var $params */
        $params = array();
        $params['nrorg']        = $nrorg;
        $params['cdtpbaixapag'] = $cdtpbaixapag;

        /** @var Tipobaixapag $tipobaixapag */
        $tipobaixapag = $this->entityManager->getRepository(Tipobaixapag::class)->findOneBy($params);

        return $tipobaixapag;
    }
    
    /**
     * Insere Movimentação Bancária
     * @param $nrorg
     * @param $nrmovbanc
     * @param $cdreducont
     * @param $dtmovbanc
     * @param $vrmovbanc
     * @return Movbancario
     */
    public static function insereMovBancario(
        $nrorg, $nrmovbanc, $cdreducont, $dtmovbanc, $vrmovbanc
    ){
        /** @var Movbancario $movbancario */
        $movbancario = new Movbancario;
        $movbancario->setNrorg($nrorg);
        $movbancario->setCdreducont($cdreducont);
        $movbancario->setNrmovbanc($nrmovbanc);
        $movbancario->setDtmovbanc($dtmovbanc);
        $movbancario->setVrmovbanc($vrmovbanc);
        $movbancario->setIdmovbanc('D');
        $movbancario->setIdorigmbanc('P');
        $movbancario->setIdstatusctbbanc('P');
        $movbancario->setDshismbanc('BaixaPag');
        $movbancario->setDtinclusao(DateUtil::getDataAtual());

        return $movbancario;
    }

    /**
     * Insere Baixa Paga
     * @param $nrorg
     * @param $cdfilial
     * @param $cdempresa
     * @param $cdoperador
     * @param $cdtpbaixapag
     * @param $cdreducont
     * @param $dtlancbaixap
     * @param $nrmovbanc
     * @param $nrbaixapag
     * @param $idstatusbpag
     * @param $dsbaixapag
     * @param $dtbaixapag
     * @param $vrbaixapag
     * @return Baixapaga
     */
    public static function insereBaixapaga(
        $nrorg, $cdfilial, $cdempresa, $cdoperador, $cdtpbaixapag, $cdreducont, $dtlancbaixap, $nrmovbanc, $nrbaixapag,
        $idstatusbpag, $dsbaixapag, $dtbaixapag, $vrbaixapag, $nrcheque, $dsnomcheque, $idstemicheq
    ){
        /** @var Baixapaga $baixapaga */
        $baixapaga = new Baixapaga();
        $baixapaga->setNrorg($nrorg);
        $baixapaga->setCdfilial($cdfilial);
        $baixapaga->setNrorginclusao($nrorg);
        $baixapaga->setCdempresa($cdempresa);
        $baixapaga->setNrmovbanc($nrmovbanc);
        $baixapaga->setNrbaixapag($nrbaixapag);
        $baixapaga->setDtbaixapag($dtbaixapag);
        $baixapaga->setCdreducont($cdreducont);
        $baixapaga->setDsbaixapag($dsbaixapag);
        $baixapaga->setVrbaixapag($vrbaixapag);
        $baixapaga->setCdoperador($cdoperador);
        $baixapaga->setDtlancbaixap($dtlancbaixap);
        $baixapaga->setIdstatusbpag($idstatusbpag);
        $baixapaga->setCdtpbaixapag($cdtpbaixapag);
        $baixapaga->setCdoperinclusao($cdoperador);
        $baixapaga->setNrcheque($nrcheque);
        $baixapaga->setDsnomcheque($dsnomcheque);
        $baixapaga->setIdstemicheq($idstemicheq);
        $baixapaga->setDtinclusao(DateUtil::getDataAtual());

        return $baixapaga;
    }

    /**
     * Atualiza Conta a Pagar
     * @param Contapagar $contapagar
     * @param $idstatpag
     * @param $idmovfluxo
     * @param $dtmovfluxo
     * @return Contapagar
     */
    public static function atualizaContapagar(
        Contapagar $contapagar, $idstatpag, $idmovfluxo, $dtmovfluxo
    ){
        $contapagar->setIdstatpag($idstatpag);
        $contapagar->setIdmovfluxo($idmovfluxo);
        $contapagar->setDtmovfluxo($dtmovfluxo);

        return $contapagar;
    }

    /**
     * Atualiza Cheque
     * @param Cheque $cheque
     * @param $idstcheque
     * @param $dsnomcheq
     * @param $dscheque
     * @param $dtmovbancheq
     * @param $dtemischeq
     * @param $vrcheque
     * @param $cdmoticanc
     * @return Cheque
     */
    public static function atualizaCheque(
        Cheque $cheque, $idstcheque, $dsnomcheq, $dscheque, $dtmovbancheq, $dtemischeq, $vrcheque, $cdmoticanc = null
    ){
        $cheque->setIdstcheque($idstcheque);
        $cheque->setDsnomcheq($dsnomcheq);
        $cheque->setDscheque($dscheque);
        $cheque->setDtmovbancheq($dtmovbancheq);
        $cheque->setDtemischeq($dtemischeq);
        $cheque->setVrcheque($vrcheque);
        $cheque->setCdmoticanc($cdmoticanc);

        return $cheque;
    }

    /**
     * Valida Adiantamento
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixapag
     * @return array
     */
    public function validaAdiantamento(
        $nrorg, $cdfilial, $nrbaixapag
    ){
        /** @var $params */
        $params = array();
        $params['P_NRORG'     ] = $nrorg;
        $params['P_CDFILIAL'  ] = $cdfilial;
        $params['P_NRBAIXAPAG'] = $nrbaixapag;

        $sql = "SELECT A.NRBAIXAPAG FROM (SELECT MAX(BO.NRBAIXAPAG) NRBAIXAPAG
                  FROM BOLETPAGAR  BO, CONTAPAGAR CR, ADIANTAPAGAR AR, TIPOCTPAGAR TR
                 WHERE (BO.NRORG         = :P_NRORG       ) AND
                       (BO.CDFILIBAIPAG  = :P_CDFILIAL    ) AND
                       (BO.NRBAIXAPAG    = :P_NRBAIXAPAG  ) AND
                       (CR.CDFILIBPAG  = BO.CDFILIAL    ) AND
                       (CR.NRLANBOLPAG   = BO.NRLANBOLPAG ) AND
                       (CR.IDSTATPAG     = 'B'            ) AND
                       (CR.CDFILIAL      = AR.CDFILIAL    ) AND
                       (CR.NRLANCTONF    = AR.NRLANCTONF  ) AND
                       (CR.NRSEQVENPAG   = AR.NRSEQVENPAG ) AND
                       (CR.NRPARCELAPAG  = AR.NRPARCELAPAG) AND
                       (TR.CDTPCTPAGAR   = CR.CDTPCTPAGAR ) AND
                       (TR.IDTPCTPAGAR = '2'            ))A";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Valida Adiantamento
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixapag
     * @return array
     */
    public function validaAdiantamentoMultiploTitulos(
        $nrorg, $cdfilial, $nrlanbolpag
    ){
        /** @var $params */
        $params = array();
        $params['P_NRORG'     ] = $nrorg;
        $params['P_CDFILIAL'  ] = $cdfilial;
        $params['P_NRLANBOLPAG'] = $nrlanbolpag;
        
        $sql = "SELECT
                    A.NRBAIXAPAG,
                    A.NRORG
                FROM
                    (
                        SELECT
                            MAX(BO.NRBAIXAPAG) NRBAIXAPAG,
                            CP.NRORG
                        FROM
                            BOLETPAGAR   BO,
                            CONTAPAGAR   CP,
                            ADIANTAPAGAR AP,
                            TIPOCTPAGAR  TP,
                            BAIXAPAGA    BA
                        WHERE
                            BO.NRORG         = :P_NRORG
                            AND BO.CDFILIBAIPAG = BA.CDFILIAL
                            AND BO.NRBAIXAPAG = BA.NRBAIXAPAG
                            AND ( BO.CDFILIAL = :P_CDFILIAL )
                            AND ( BO.NRLANBOLPAG = :P_NRLANBOLPAG)
                            AND CP.CDFILIBPAG = BO.CDFILIAL
                            AND CP.NRLANBOLPAG = BO.NRLANBOLPAG
                            AND CP.IDSTATPAG = 'B'
                            AND CP.CDFILIAL = AP.CDFILIAL
                            AND CP.NRLANCTONF = AP.NRLANCTONF
                            AND CP.NRSEQVENPAG = AP.NRSEQVENPAG
                            AND CP.NRPARCELAPAG = AP.NRPARCELAPAG
                            AND TP.CDTPCTPAGAR = CP.CDTPCTPAGAR
                            AND TP.IDTPCTPAGAR = '2'
                        GROUP BY
                            CP.NRORG,
                            BO.NRBAIXAPAG
                    ) A";
                    
        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }

    /**
     * Valida Periodo Contabil
     * @param $nrorg
     * @param $cdfilial
     * @param $dtperiodo
     * @return array
     */
    public function validaPeriodoContabil(
        $nrorg, $cdfilial, $dtperiodo
    ){
        /** @var $params */
        $params = array();
        $params['P_NRORG'    ] = $nrorg;
        $params['P_CDEMPRESA'] = $cdfilial;
        $params['P_DTPERIODO'] = $dtperiodo;

        $sql = "SELECT CO.CDEMPRESA, CO.DTINICCTBL, CO.DTFINACCTBL
                  FROM CONTOPECTBL CO
                 WHERE CO.NRORG     = :P_NRORG     AND
                       CO.CDEMPRESA = :P_CDEMPRESA AND
                       :P_DTPERIODO BETWEEN CO.DTINICCTBL AND CO.DTFINACCTBL";

        $std = $this->connection->prepare($sql);
        $std->execute($params);
        return $std->fetchAll();
    }
    
    /**
     * Executa Procedure GRAVA_BAIXA_IMPOSTO
     * @param $cdfilial
     * @param $nrbaixa
     * @param $cdreducont
     * @param $nrmovidebi
     * @param $idoperacao
     */
    public function execGravaBaixaImposto(
        $cdfilial, $nrbaixa, $cdreducont, $nrmovidebi, $idoperacao
    ){
        $procedure = new StoredProcedure($this->entityManager->getConnection(), 'GRAVA_BAIXA_IMPOSTO');
        $procedure->addParam(new Param('P_CDFILIAL'  , Param::PARAM_INPUT, $cdfilial   , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRBAIXA'   , Param::PARAM_INPUT, $nrbaixa    , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_CDREDUCONT', Param::PARAM_INPUT, $cdreducont , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_NRMOVIDEBI', Param::PARAM_INPUT, $nrmovidebi , Param::PARAM_TYPE_STR));
        $procedure->addParam(new Param('P_IDOPERACAO', Param::PARAM_INPUT, $idoperacao , Param::PARAM_TYPE_STR));

        $procedure->execute();
    }
    
    
    
    /**
     * Busca Impbaixapag Baixa
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixapag
     * @return Empresa
     */
    public function buscaImpbaixapagBaixa($nrorg, $cdfilial, $nrbaixapag){
        /** @var $params */
        $params = array();
        
        $params['nrorg']      = $nrorg;
        $params['cdfilial']   = $cdfilial;
        $params['nrbaixarec'] = $nrbaixapag;

        /** @var Impbaixapag $impbaixapag */
        $impbaixapag = $this->entityManager->getRepository(Impbaixarec::class)->findBy($params);

        return $impbaixapag;
    }
    
    /**
     * Busca Impbaixapag Titulo
     * @param $nrorg
     * @param $cdfilial
     * @param $nrbaixapag
     * @param nrlanbolpag
     * @return Empresa
     */
    public function buscaImpbaixapagTitulo($nrorg, $cdfilial, $nrbaixapag, $nrlanbolpag){
        /** @var $params */
        $params = array();
        
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrbaixapag']  = $nrbaixapag;
        $params['nrlanbolrec'] = $nrlanbolpag;

        /** @var Impbaixapag $impbaixapag */
        $impbaixapag = $this->entityManager->getRepository(Impbaixarec::class)->findBy($params);

        return $impbaixapag;
    }
    
    

    /**
     * Busca Ajuste(s) Boleto a Pagar
     * @param $nrorg
     * @param $cdfilial
     * @param $nrlanbolpag
     * @return Ajbolpag
     */
    public function buscaAjsbolpag($nrorg, $cdfilial, $nrlanbolpag){
        /** @var $params */
        $params = array();
        $params['nrorg']       = $nrorg;
        $params['cdfilial']    = $cdfilial;
        $params['nrlanbolpag'] = $nrlanbolpag;

        /** @var Ajbolpag $ajsbolpag */
        $ajsbolpag = $this->entityManager->getRepository(Ajbolpag::class)->findBy($params);

        return $ajsbolpag;
    }

    /**
     * Busca Tipo de Ajuste
     * @param $nrorg
     * @param $cdajuctpag
     * @return Tipoajpagar
     */
    public function buscaTipoajpagar($nrorg, $cdajuctpag){
        /** @var $params */
        $params = array();
        $params['nrorg']      = $nrorg;
        $params['cdajuctpag'] = $cdajuctpag;

        /** @var Ajbolpag $ajbolpag */
        $tipoajpagar = $this->entityManager->getRepository(Tipoajpagar::class)->findOneBy($params);

        return $tipoajpagar;
    }
    
    /**
     * Busca GeralPag
     * @param $nrorg
     * @return GeralPag
     */
    public function buscaGeralPag($nrorg){
        /** @var $params */
        $params = array();
        $params['nrorg'] = $nrorg;

        /** @var GeralPag $geralpag */
        $geralpag = $this->entityManager->getRepository(GeralPag::class)->findOneBy($params);

        return $geralpag;
    }

}