<?php
namespace Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco;

use Teknisa\Libs\Util\DateUtil;
use Teknisa\FIN\Modules\ContaReceber\Services\Emissao\GeraEmissaoBoletoService;

/**
 * Class GeraEmissaoBoletoSicrediService
 * @package Teknisa\FIN\Modules\ContaReceber\Services\Emissao\Banco
 */
class GeraEmissaoBoletoSicrediService extends GeraEmissaoBoletoService
{

    /**
     * Tamanho do operador da linha digitavel de nosso numero.
     */
    const TAMANHO_NOSSO_NUMERO                   = 5;
    const MODULO_DIGITO_VERIFICADOR_NOSSONUM     = 10;
    const MODULO_DIGITO_VERIFICADOR_LINHADIGIT   = 10;
    const FATOR_VENCIMENTO                       = '07/10/1997';
    const MODULO_DIGITO_VERIFICADOR = 11;

    public function getNossoNumeroComDigito()
    {
        $codigoReduzido = $this->getCodigoReduzido();
        $cdcontador      = self::PREFIXO_CDCONTADOR.$codigoReduzido;
        $nossoNumero     = \Geral\Util\NovoCodigo::geraCodigo (
            $this->boleto->getNrorg(), $cdcontador, 1, self::TAMANHO_NOSSO_NUMERO
        );
        $cdagencia = substr($this->getCdAgencia(), 0, 4);
        $posto_cedente = substr($this->getCdcedenteban(),0,7);
        $now = DateUtil::getDataAtual();
        $nossoNumerocalc = $cdagencia.$posto_cedente.substr(DateUtil::getAno($now), -2) . '2' . substr($nossoNumero, -5);
        return substr(DateUtil::getAno($now), -2) . '2'.$nossoNumero.$this->getDigitoVerificadorNossoNumero($nossoNumerocalc);
        
    }


    /** Função que monta a Linha Digitável do boleto de acordo com o Layout do Banco Itaú.
     * @param $nossoNumero
     * @return string
     */

    public function getLinhaDigitavel($nossoNumero)
    {
        $cdagencia = substr($this->getCdAgencia(), 0, 4);
        $cdcontcorr = substr($this->getCdcontcorr(), 0, 5);
        // $nossoNumero = str_pad(substr($nossoNumero, 0, 9), 9, 0, STR_PAD_LEFT);
        $cdcartcobbol = str_pad(substr($this->boleto->getCdcartcobbol(), 0, 1), 1, 0, STR_PAD_LEFT);
        $idformcadbrec = str_pad($this->boleto->getIdformcadbrec(), 1, 0, STR_PAD_LEFT);
         
        $grupo1  = $this->getIdentificadorBanco();
        $grupo1 .= self::MOEDA_REAL;
        $grupo1 .= $idformcadbrec;
        $grupo1 .= $cdcartcobbol;
        $grupo1 .= substr($nossoNumero,0,3);
        $grupo1 .= $this->getVerificadorGruposLinhaDigitavel($grupo1);
        
        $grupo2  = substr($nossoNumero,3,6);
        $grupo2 .= substr($cdagencia, 0, 4);
        $grupo2 .= $this->getVerificadorGruposLinhaDigitavel($grupo2);

        //posto + conta corrente
        $grupo3 = str_pad(substr($this->getCdcedenteban(),0,7), 7, 0,STR_PAD_LEFT);
        $grupo3 .= "1";
        $grupo3 .= "0";
        
        // DV do campo livre
        $campolivre  = $idformcadbrec;
        $campolivre .= $cdcartcobbol;
        $campolivre .= $nossoNumero;
        $campolivre .= $cdagencia;
        $campolivre .= str_pad(substr($this->getCdcedenteban(),0,7), 7, 0,STR_PAD_LEFT);
        $campolivre .= '1';
        $campolivre .= '0';
        
        $grupo3 .= $this->getDigitoVerificador($campolivre);
        $grupo3 .= $this->getVerificadorGruposLinhaDigitavel($grupo3);

        $grupo4 = $this->getDigitoCodigoDeBarras($nossoNumero);

        $grupo5 = $this->getFatorVencimento() . $this->getValorNominal();

        return self::concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5);
    }

    public function getCodigoDeBarras($nossoNumero)
    {
        $cdagencia = substr($this->getCdAgencia(), 0, 4);
        $cdcontcorr = substr($this->getCdcontcorr(), 0, 5);
        $nossoNumero = str_pad(substr($nossoNumero, 0, 9), 9, 0, STR_PAD_LEFT);
        $cdcartcobbol = str_pad(substr($this->boleto->getCdcartcobbol(), 0, 1), 1, 0, STR_PAD_LEFT);
        $idformcadbrec = str_pad($this->boleto->getIdformcadbrec(), 1, 0, STR_PAD_LEFT);

        $codigoDeBarras  = $this->getIdentificadorBanco();
        $codigoDeBarras .= self::MOEDA_REAL;
        $codigoDeBarras .= "_SUBSTITUICAO_DV_";
        $codigoDeBarras .= $this->getFatorVencimento();
        $codigoDeBarras .= $this->getValorNominal();
        
        $campolivre  = $idformcadbrec;
        $campolivre .= $cdcartcobbol;
        $campolivre .= $nossoNumero;
        $campolivre .= $cdagencia;
        $campolivre .= str_pad(substr($this->getCdcedenteban(),0,7), 7, 0,STR_PAD_LEFT);
        $campolivre .= '1';
        $campolivre .= '0';
        $campolivre .= $this->getDigitoVerificador($campolivre);
        
        $codigoDeBarras .= $campolivre;

        $digito         = $this->getDigitoVerificador(str_replace("_SUBSTITUICAO_DV_", "", $codigoDeBarras));
        $digitoCodigoBarras = $digito;
        $codigoDeBarras = str_replace("_SUBSTITUICAO_DV_", $digito, $codigoDeBarras);

        return $codigoDeBarras;
    }
    
    public function getDigitoCodigoDeBarras($nossoNumero)
    {
        $cdagencia = substr($this->getCdAgencia(), 0, 4);
        $cdcontcorr = substr($this->getCdcontcorr(), 0, 5);
        $nossoNumero = str_pad(substr($nossoNumero, 0, 9), 9, 0, STR_PAD_LEFT);
        $cdcartcobbol = str_pad(substr($this->boleto->getCdcartcobbol(), 0, 1), 1, 0, STR_PAD_LEFT);
        $idformcadbrec = str_pad($this->boleto->getIdformcadbrec(), 1, 0, STR_PAD_LEFT);

        $codigoDeBarras  = $this->getIdentificadorBanco();
        $codigoDeBarras .= self::MOEDA_REAL;
        $codigoDeBarras .= "_SUBSTITUICAO_DV_";
        $codigoDeBarras .= $this->getFatorVencimento();
        $codigoDeBarras .= $this->getValorNominal();
        
        $campolivre  = $idformcadbrec;
        $campolivre .= $cdcartcobbol;
        $campolivre .= $nossoNumero;
        $campolivre .= $cdagencia;
        $campolivre .= str_pad(substr($this->getCdcedenteban(),0,7), 7, 0,STR_PAD_LEFT);
        $campolivre .= '1';
        $campolivre .= '0';
        $campolivre .= $this->getDigitoVerificador($campolivre);
        
        $codigoDeBarras .= $campolivre;

        $digito = $this->getDigitoVerificador(str_replace("_SUBSTITUICAO_DV_", "", $codigoDeBarras));

        return $digito;
    }

    private function getValorNominal()
    {
        $valorBoleto = (string)$this->getValorBoleto();
        $valorBoleto = str_replace(',','.',$valorBoleto);
        $valorBoleto = number_format($valorBoleto, 2, '.', '');
        $arrayExclusao = array('.', ',');
        $valorBoleto = str_replace($arrayExclusao, "", $valorBoleto);
        $valorBoleto = str_pad($valorBoleto, 10, "0", STR_PAD_LEFT);
        return $valorBoleto;
    }

    private function getCodigoCedente()
    {
        if ($this->getCdcedenteban() == false) {
            $codigoCedente = '0000000';
        } else {
            $codigoCedente = substr($this->getCdcedenteban(), 0, 7);
        }

        return $codigoCedente;
    }

    private function getIdentificadorBanco()
    {
        $identificadorBanco = $this->getCdbanccnab();
        return $identificadorBanco;
    }

    private function getFatorVencimento()
    {
        $datavenc            = DateUtil::truncateData($this->getDataVencimento());
        $dataFatorVencimento = DateUtil::truncateData(DateUtil::getDataDeString(self::FATOR_VENCIMENTO));
        $fatorDeVencimento   = DateUtil::getQtdeDiasEntre($dataFatorVencimento, $datavenc);
        $fatorDeVencimento   = str_pad($fatorDeVencimento, 4, "0", STR_PAD_LEFT);
        return $fatorDeVencimento;
    }

    private function getVerificadorGruposLinhaDigitavel($linhaDigitavel)
    {
        $linhaDigitavel = strrev($linhaDigitavel);
        $peso = 2;
        $total = 0;
        for($i = 0; $i<=strlen($linhaDigitavel);$i++){
            $result = (int) substr($linhaDigitavel,$i,1)* $peso;
            if ($result > 9){
                $total += (int)substr((string)$result, 0, 1) + (int)substr((string)$result, 1,1);
            }else{
                $total += ((int) substr($linhaDigitavel,$i,1)* $peso);
            }
            $peso = $this->incrementaValorPeso($peso, 1, 2);

        }
        
        $multiplo = 10;
        
        while($multiplo < $total){
            $multiplo += 10;
        }
        
        $digito = $multiplo - $total;
        
        return $digito;
    }

    public function getRelatorioBoleto($nrorg, $cdreducont){

        /** @var $contcorr \Geral\Entities\Contcorr */
        $contcorr = \Geral\Entities\Contcorr::findOneByNrorgAndCdreducont($nrorg,$cdreducont);
        $contcorr->getcdbanco();

        /** @var $dsarqbolcomp Banco*/
        $dsarqbolcomp = \Geral\Entities\Banco::findOneByCdbanco($contcorr->getcdbanco());
        $string = $dsarqbolcomp->getDsarqbolcomp();
        $tam = strlen($string);
        $tam_b = strrpos($string,'\\');
        $ext = -($tam - strrpos($string,'.'));
        if($tam_b === false){
            return NULL;
        }
        else
            return ($string_p = substr($string, ++$tam_b, $ext));
    }

    private function concatLinhaDigitavel($grupo1,$grupo2,$grupo3,$grupo4,$grupo5){

        $linhadigitavel = substr($grupo1,0,5) . '.' . substr($grupo1, 5) . ' ';

        $linhadigitavel .= substr($grupo2,0, 5) . '.' . substr($grupo2, 5) . ' ';

        $linhadigitavel .= substr($grupo3,0, 5) . '.' . substr($grupo3, 5) . ' ';

        $linhadigitavel .= $grupo4 . ' ';

        $linhadigitavel .= $grupo5;

        return $linhadigitavel;
    }

    private function getDigitoVerificadorNossoNumero($sequencia){

        // Inverte o nosso numero.
        $sequencia   = strrev($sequencia);
        $peso = 2;
        $acumulador = 0;

        // Este loop percorrera todos os caracteres da variavel $nossonumero
        for ($i = 0; $i < strlen($sequencia); $i++) {
            $result = ((int)substr($sequencia, $i, 1) * $peso);
            $acumulador += $result;
            $peso = $this->incrementaValorPeso($peso, 2, 9);
        }
        
        $digito = $acumulador % 11;
        $digito = self::MODULO_DIGITO_VERIFICADOR - $digito;
        $digito = $this->avaliarDigitoRetornoNossoNumero($digito);
        $this->setDigitoCodigoBarras($digito);

        return $digito;
    }

    protected function avaliarDigitoRetornoNossoNumero($digito)
    {
        if ($digito > 9) return 0;
        else return $digito;
    }

    public function chamaRelatorio()
    {
        // TODO: Implement chamaRelatorio() method.
    }
}
