<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services;

use Doctrine\ORM\EntityManager;
use Financeiro\Entities\Boletreceber;
use Financeiro\Entities\Cnabrec;
use Geral\Entities\Endeclie;
use Geral\Entities\Cliente;
use Teknisa\Libs\Util\DateUtil;
use Geral\Util\NovoCodigo;
use Indeterminado\Entities\Logboletoremrec;
use Teknisa\FIN\Modules\ContaReceber\Factory\RemessaReceberFactory ;
use Teknisa\FIN\Modules\ContaReceber\Factory\TitulosAReceberFactory;
use Teknisa\FIN\Modules\ContaReceber\Services\Remessa\GeraCnabService;
//use Zeedhi\FileServerAPI\API;

/**
 * Class RemessaReceberService
 * @package Teknisa\FIN\Modules\ContaReceber\Services
 */
class RemessaReceberService {

    protected $entityManager;
    protected $remessaReceberFactory;
    protected $titulosAReceberFactory;
    protected $server;

    const CUSTOM_FOLDER   = 'erp/modules/fin/modules/contaReceber/cnab/remessa';
    const FILE_FIELD_NAME = 'LOCAL_ARQUIVO';
    const ID_STATUS_BOLETO_ENVIADO     = '10';
    const CODIGO_COMPENSACAO_SANTANDER = '033';
    const CODIGO_COMPENSACAO_BRADESCO  = "237";
    const CODIGO_COMPENSACAO_ITAU      = "341";
    const CODIGO_COMPENSACAO_BRASIL    = "001";
    const CODIGO_COMPENSACAO_CAIXA     = "104";
    const CODIGO_COMPENSACAO_SICOOB    = "756";
    const CODIGO_COMPENSACAO_SICREDI   = "748";
    const CODIGO_COMPENSACAO_SOFISA   = "637";
    const CODIGO_COMPESACAO_VOTORANTIM   = "655";
    const CODIGO_COMPENSACAO_DAYCOVAL   = "707";
    const CODIGO_COMPENSACAO_BANRISUL   = "041";
    const ID_BOLETO_RECEBER_ALTERADO   = '1';

    /**
     * @param EntityManager $entityManager
     * @param RemessaReceberFactory $remessaReceberFactory
     */
    public function __construct(EntityManager $entityManager, RemessaReceberFactory $remessaReceberFactory, TitulosAReceberFactory $titulosAReceberFactory, $server)    {
        $this->entityManager          = $entityManager;
        $this->remessaReceberFactory  = $remessaReceberFactory;
        $this->titulosAReceberFactory = $titulosAReceberFactory;
        $this->server = $server;
    }

    /**
     * Gera Remessa Receber
     * @param $nrorg
     * @param $cdfilial
     * @param $remessa
     * @param $boletos
     * @return null|string
     * @throws \Exception
     */
    public function geraRemessaReceber(
        $nrorg, $cdfilial, $remessa, $boletos
    ){

        $connection = $this->entityManager->getConnection();
        $connection->beginTransaction();
        try {
            // Variáveis de Inicialização
            $msg_sem_endereco = '<br/><br/>';
            $msg_sem_bairro   = '<br/><br/>';
            $sem_endereco     = false;
            $sem_bairro       = false;
            $vrtotbolcnabrec  = 0;
            $qtbolenvcnabrec  = 0;
            
            // Percorre Boletos Receber
            foreach ($boletos as $boleto) {
                // ENDEREÇO DE COBRANÇA OBRIGATÓRIO
                /** @var Endeclie $endeclie */
                $endeclie = Endeclie::findOneByNrorgAndCdclienteAndIdtpendeclie(
                    $nrorg, $boleto['CDCLIENTE'], 'C'
                );
                
                /** @var Cliente $cliente */
                $cliente = Cliente::findOneByNrorgAndCdcliente(
                    $nrorg, $boleto['CDCLIENTE']
                );
                
                if($endeclie){
                    if(!$endeclie->getNmbairclie()){
                        $msg_sem_bairro .= $cliente->getNmrazsocclie().'<br/>';
                        $sem_bairro      = true;
                    }
                }else{
                    $msg_sem_endereco .= $cliente->getNmrazsocclie().'<br/>';
                    $sem_endereco      = true;
                }
            }
            
            if($sem_endereco)
                throw new \Exception('Operação bloqueado. Existe(m) cliente(s) sem endereço de cobrança cadastrado: '.$msg_sem_endereco);
            
            if($sem_bairro)
                throw new \Exception('Operação bloqueado. Existe(m) cliente(s) sem bairro cadastrado: '.$msg_sem_bairro);
            
            // Percorre Boletos Receber
            foreach ($boletos as $boleto) {
                // Soma Valores Líquidos Boleto Receber
                $vrtotbolcnabrec += $boleto['VRFINALBREC'];

                // Soma Número de Boletos
                $qtbolenvcnabrec += 1;
            }

            // Gera Novo Código CNAB Receber
            $nrcnabrec = NovoCodigo::geraCodigo($nrorg, 'CNABREC'.$remessa['CDEMPRESA'], 1, 10);

            // Gera Novo Código Sequencial Remessa
            $nrseqremessarec = NovoCodigo::geraCodigo($nrorg, 'SEQREMESSAREC'.$remessa['CDREDUCONT'], 1, 10);

            // Monta Nome do Arquivo CNAB Receber
            $nmarqcnabrec = 'E'.$nrorg.$remessa['CDEMPRESA'].$nrcnabrec.'.REM';

            // Formata Data CNAB
            $dtcnabrec = self::formataDataHora($remessa['DTCNABREC']);

            // Verifica Existência Conta Cobrança
            $cdreducontcobr = $remessa['CDREDUCONTCOBR']?:$remessa['CDREDUCONT'];

            // Insere Remessa Receber
            $cnabrec = $this->remessaReceberFactory->insereCnabRec(
                $nrorg, $remessa['CDEMPRESA'], $nrcnabrec, $remessa['CDOPERADOR'],
                $dtcnabrec, $remessa['IDTIPCNABREC'], $remessa['CDREDUCONT'],
                $remessa['CDBANCCNAB'], $cdreducontcobr, $remessa['IDOPECNABREC'],
                $remessa['QTBOLENVCNABREC'], $remessa['VRTOTBOLCNABREC'], $nrseqremessarec,
                $remessa['DSARQCNABREC'], $remessa['CDFILIAL']
            );

            $this->entityManager->persist($cnabrec);
            $this->entityManager->flush();
            
            // Percorre Boletos Receber
            foreach ($boletos as $boleto) {

                /** @var Boletreceber $boletreceber */
                $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                    $boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $nrorg
                );

                // Atualiza Bolet Receber
                $boletreceber = $this->remessaReceberFactory->atualizaBoletreceber(
                    $boletreceber, $cnabrec->getNrcnabrec(), self::ID_STATUS_BOLETO_ENVIADO, self::ID_BOLETO_RECEBER_ALTERADO
                );

                $this->entityManager->persist($boletreceber);
                $this->entityManager->flush();
                
                /** @var Logboletoremrec $logboletoremrec */
                $logboletoremrec = $this->remessaReceberFactory->buscaLogboletoremrec(
                    $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['CDEMPRESA'], $nrcnabrec
                );
                
                if(!$logboletoremrec) {
                    // Insere Log de Atualização Boleto Remessa Receber
                    /** @var Logboletoremrec $logboletoremrec */
                    $logboletoremrec = $this->remessaReceberFactory->insereLogBoletoRem(
                        $nrorg, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $nrcnabrec,
                        $boleto['CDEMPRESA'], $remessa['CDOPERADOR']
                    );
                    $this->entityManager->persist($logboletoremrec);
                }
                
                $this->entityManager->flush();
            }
            $this->entityManager->flush();
            
            $remessa['CDCONTCORRCOBR'] = isset($remessa['CDCONTCORRCOBR'])?$remessa['CDCONTCORRCOBR']:null;

            // Verifica Banco Cnab Receber
            self::verificaBancoCnabRec(
                $boletos, $cnabrec, $remessa['DSARQCNABREC'], $nrorg, $remessa['CDBANCCNAB'], $remessa['CDEMPRESA'], $remessa['IDTIPCNABREC'], $nrcnabrec,
                $remessa['IDOPECNABREC'], $remessa['CDCONTCORR'], $remessa['CDCONTCORRCOBR'], $remessa['CDREDUCONT'], $nrcnabrec, $nmarqcnabrec,
                $this->entityManager, $this->titulosAReceberFactory
            );

            // Upload Arquivo Remessa Receber
            //$dsarqcnabrec = self::uploadArqRemessaReceber($remessa['DSARQCNABREC'], $nmarqcnabrec);
            $dsarqcnabrec = $remessa['DSARQCNABREC'].$nmarqcnabrec;

            // Atualiza Local Arquivo CNAB Receber
            self::atualizaLocalArqCnabRec($nrorg, $remessa['CDEMPRESA'], $nrcnabrec, $dsarqcnabrec);

            $this->entityManager->flush();
            $connection->commit();

            return $dsarqcnabrec;
        }catch(\Exception $e){
            $connection->rollback();
            throw $e;
            return null;
        }
    }

    /**
     * @param $local_arquivo
     */
    public function downloadRemessaReceber(
        $local_arquivo
    ){
        $nome_arquivo =  substr($local_arquivo, strrpos($local_arquivo, '/')+1);

        header('Accept-Ranges: none');
        header("Content-Type: text/plain");
        header("Content-Disposition: attachment; filename=" . $nome_arquivo);

        readfile($local_arquivo);
        exit;
    }

    /**
     * Atualiza Local Arquivo CNAB Receber[File Server]
     * @param $nrorg
     * @param $cdempresa
     * @param $nrcnabrec
     * @param $dsarqcnabrec
     * @return Cnabrec
     */
    public function atualizaLocalArqCnabRec(
        $nrorg, $cdempresa, $nrcnabrec, $dsarqcnabrec
    ){

        /** @var Cnabrec $cnabrec */
        $cnabrec = $this->remessaReceberFactory->buscaCnabrec(
            $nrorg, $cdempresa, $nrcnabrec
        );

        /** @var Cnabrec $cnabrec */
        $cnabrec = $this->remessaReceberFactory->atualizaCnabrec(
            $cnabrec, $dsarqcnabrec
        );

        return $cnabrec;
    }

    /**
     * Verifica Banco CNAB Receber
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $cdbanco
     */
    public static function verificaBancoCnabRec(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
        $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
        EntityManager $entitymanager, Titulosareceberfactory $titulosareceberfactory
    ){
        $codigoCompensacao = $cnabrec->getReducont()->getAgencia()->getBanco()->getCdbanccnab();
        if($codigoCompensacao === self::CODIGO_COMPENSACAO_SANTANDER){
            self::geraCnabSantander(
                $boletos, $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
                $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
                $entitymanager, $titulosareceberfactory
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_BRADESCO) {
            self::geraCnabBradesco(
                $boletos, $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec,
                $idoperacao, $cdreducont, $nmarquivo,
                $entitymanager, $titulosareceberfactory
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_ITAU) {
            self::geraCnabItau(
                $boletos, $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec,
                $idoperacao, $cdreducont, $nmarquivo,
                $entitymanager, $titulosareceberfactory
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_BRASIL) {
            self::geraCnabBrasil(
                $boletos, $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
                $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
                $entitymanager, $titulosareceberfactory
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_CAIXA) {
            self::geraCnabCaixa(
                $boletos, $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec,$nrcnabrec,
                $idoperacao,$cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
                $entitymanager, $titulosareceberfactory
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_SICOOB) {
            self::geraCnabSicoob(
               $boletos, $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
                $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
                $entitymanager, $titulosareceberfactory
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_SICREDI) {
            self::geraCnabSicredi(
                $boletos, $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec,
                $idoperacao, $cdreducont, $nmarquivo,
                $entitymanager, $titulosareceberfactory
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_SOFISA) {
            self::geraCnabSofisa(
                $boletos, $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec,
                $idoperacao, $cdreducont, $nmarquivo,
                $entitymanager, $titulosareceberfactory
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPESACAO_VOTORANTIM) {
            self::geraCnabVotorantim(
                $boletos, $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
                $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
                $entitymanager, $titulosareceberfactory
            );
        }
        elseif ($codigoCompensacao === self::CODIGO_COMPENSACAO_DAYCOVAL) {
            self::geraCnabDaycoval(
                $boletos, $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec,
                $idoperacao, $cdreducont, $nmarquivo,
                $entitymanager, $titulosareceberfactory
            );
        }
        elseif($codigoCompensacao === self::CODIGO_COMPENSACAO_BANRISUL){
            self::geraCnabBanrisul(
                $boletos, $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
                $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
                $entitymanager, $titulosareceberfactory
            );
        }
    }

    /**
     * Gera Cnab Santander
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $caminho
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrcnabrec
     * @param $idoperacao
     * @param $cdcontacorrente
     * @param $cdcontcorrcobranca
     * @param $cdreducont
     * @param $nrcnabrec
     * @param $nmarquivo
     */
    public static function geraCnabSantander(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
        $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
        EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){
        $emiteremessa = GeraCnabService::factory($cnabrec, $entitymanager, $titulosareceberfactory);
        if (count($boletos) != 0){
            $contreg  = 3;
            $contador = 1;
            $emiteremessa->geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, null, null, $contreg, $cnabrec->getCdFilial(), $contador);
            $emiteremessa->geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont,null,$cnabrec->getCdFilial());
            foreach ($boletos as $boleto) {

                $emiteremessa->geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(),
                    $nrcnabrec, $idoperacao, $boleto['CDAGENCIA'], $cdcontacorrente, $cdcontcorrcobranca, $boleto['CDNOSNUMREC'],
                    $boleto['IDCARTCOBREC'], $boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $boleto['DTVENBOLREC'], $boleto['VRFINALBREC'], $boleto['VRPEMORBREC'], $boleto['VRMORDIABREC'],
                    $boleto['VRPEMULBREC'], $boleto['VRMULTABREC'], $boleto['VRFINALBREC'], $boleto['CDCARTCOBBOL'], $boleto['CDDIAPROTBOL'], $cdreducont,  $boleto['IDIMPRBOLBOL'],
                    $boleto['QTDIAPROTBOL'], $contador
                );
                $contador += 1;
                $emiteremessa->geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $boleto['CDCLIENTE'], $idoperacao, $cdempresa, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $contador);
                $contador += 1;
                $emiteremessa->geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $boleto['DTVENBOLREC'], $boleto['VRPEMULBREC'], $boleto['NRLANBOLREC'], $boleto['VRMULTABREC'], $contador);

                $contador += 1;
                $contreg  += 3;
            }
            $emiteremessa->GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco, $contreg);
            $contlote  = 1;
            $contreg  += 1;
            $emiteremessa->GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote, $contreg, $cdreducont);
        }
    }

    /**
     * Gera Cnab Sicoob
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $caminho
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrcnabrec
     * @param $idoperacao
     * @param $cdcontacorrente
     * @param $cdcontcorrcobranca
     * @param $cdreducont
     * @param $nrcnabrec
     * @param $nmarquivo
     */
    public static function geraCnabSicoob(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
        $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
        EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){
        $emiteremessa = GeraCnabService::factory($cnabrec, $entitymanager, $titulosareceberfactory);
        
        if(count($boletos) != 0){
            $contreg  = 3;
            $contador = 1;
            
            $emiteremessa->geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, $boletos[(count($boletos)-1)]['CDCARTCOBBOL'], null, $contreg, $cnabrec->getCdFilial(), $contador);
            $emiteremessa->geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, $boletos[(count($boletos)-1)]['CDCARTCOBBOL'], $cnabrec->getCdFilial());
            foreach ($boletos as $boleto) {

                $emiteremessa->geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(),
                    $nrcnabrec, $idoperacao, $boleto['CDAGENCIA'], $cdcontacorrente, $cdcontcorrcobranca, $boleto['CDNOSNUMREC'],
                    $boleto['IDCARTCOBREC'], $boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $boleto['DTVENBOLREC'], $boleto['VRFINALBREC'], $boleto['VRPEMORBREC'], $boleto['VRMORDIABREC'],
                    $boleto['VRPEMULBREC'], $boleto['VRMULTABREC'], $boleto['VRFINALBREC'], $boleto['CDCARTCOBBOL'], $boleto['CDDIAPROTBOL'], $cdreducont,  $boleto['IDIMPRBOLBOL'],
                    $boleto['QTDIAPROTBOL'], $contador
                );
                $emiteremessa->geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $boleto['CDCLIENTE'], $idoperacao, $cdempresa, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $contador);
                
                /** @var Boletreceber $boletreceber */
                $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                    $boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $boleto['NRORG']
                );
                
                $emiteremessa->geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $boleto['DTVENBOLREC'], $boleto['VRPEMULBREC'], $boleto['NRLANBOLREC'], $boleto['VRMULTABREC'], $contador);

                $contador += 1;
                $contreg  += 3;
            }
            $emiteremessa->GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco, $contreg);
            $contlote  = 1;
            $contreg  += 1;
            $emiteremessa->GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote, $contreg, $cdreducont);
        }
    }

    /**
     * Gera Cnab Bradesco
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $caminho
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $idoperacao
     * @param $cdreducont
     * @param $nmarquivo
     * @param EntityManager $entitymanager
     * @param TitulosAReceberFactory $titulosareceberfactory
     */
    public static function geraCnabBradesco(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec,
        $idoperacao, $cdreducont, $nmarquivo,
        EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){

        $emiteremessa = GeraCnabService::factory($cnabrec, $entitymanager, $titulosareceberfactory);
        
        if(count($boletos) != 0){
            $contreg  = 1;
            $contador = 1;
            $emiteremessa->geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, null, null, $contreg, $cnabrec->getCdFilial(), $contador);
            foreach($boletos as $boleto){
                $contador += 1;
                $contreg  += 1;
                
                $emiteremessa->geraTransacao($caminho, $nmarquivo,$nrorg, $cdempresa, $boleto['CDCARTCOBBOL'], $boleto['CDAGENCIA'], $boleto['CDCONTCORR'],
                    $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['VRPEMORBREC'], $boleto['IDIMPRBOLBOL'], $boleto['CDNOSNUMREC'], $boleto['DTVENBOLREC'],
                    $boleto['CDINS1COBBOL'], $boleto['QTDIAPROTBOL'], $boleto['VRMORDIABREC'], $boleto['VRFINALBREC'], $idoperacao,
                    $boleto['VRPEMULBREC'], $boleto['IDTPIJURCLIE'], $boleto['NRINSJURCLIE'], $boleto['CDCLIENTE'], $boleto['CDINS2COBBOL'], $contreg, $cnabrec->getIdtipcnabrec(), $cnabrec->getCdFilial(),$boleto['CDDIAPROTBOL'], $contador, $cdreducont,$cdbanco
                );
            }
            $contlote  = 1;
            $contreg  += 1;
            $emiteremessa->GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote, $contreg, $cdreducont);
        }
    }

    /**
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $caminho
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $idoperacao
     * @param $cdreducont
     * @param $nmarquivo
     * @param EntityManager $entitymanager
     * @param TitulosAReceberFactory $titulosareceberfactory
     */
    public static function geraCnabItau(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec,
        $idoperacao, $cdreducont, $nmarquivo,
        EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){
        $emiteremessa = GeraCnabService::factory($cnabrec, $entitymanager, $titulosareceberfactory);
        if(count($boletos) != 0){
            $contreg  = 1;
            $contador = 1;
            
            $emiteremessa->geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, null, null, $contreg, $cnabrec->getCdFilial(), $contador);
            
            foreach($boletos as $boleto){
                $contador += 1;
                $contreg  += 1;
                $emiteremessa->geraTransacao($caminho, $nmarquivo,$nrorg, $cdempresa, $boleto['CDCARTCOBBOL'], $boleto['CDAGENCIA'], $boleto['CDCONTCORR'],
                    $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['VRPEMORBREC'], $boleto['IDIMPRBOLBOL'], $boleto['CDNOSNUMREC'], $boleto['DTVENBOLREC'],
                    $boleto['CDINS1COBBOL'], $boleto['QTDIAPROTBOL'], $boleto['VRMORDIABREC'], $boleto['VRFINALBREC'], $idoperacao,
                    $boleto['VRPEMULBREC'], $boleto['IDTPIJURCLIE'], $boleto['NRINSJURCLIE'], $boleto['CDCLIENTE'], $boleto['CDINS2COBBOL'], $contreg, $cnabrec->getIdtipcnabrec(),$cnabrec->getCdFilial(),$boleto['CDDIAPROTBOL'], $contador, $cdreducont,$cdbanco
                );
            }
            $contlote  = 1;
            $contreg  += 1;
            $emiteremessa->GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote, $contreg, $cdreducont);
        }
    }
    
    /**
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $caminho
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $idoperacao
     * @param $cdreducont
     * @param $nmarquivo
     * @param EntityManager $entitymanager
     * @param TitulosAReceberFactory $titulosareceberfactory
     */
    public static function geraCnabSicredi(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec,
        $idoperacao, $cdreducont, $nmarquivo,
        EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){
        $emiteremessa = GeraCnabService::factory($cnabrec, $entitymanager, $titulosareceberfactory);
        if(count($boletos) != 0){
            $contreg  = 1;
            $contador = 1;
            
            $emiteremessa->geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, null, null, $contreg, $cnabrec->getCdFilial(), $contador);
            
            foreach($boletos as $boleto){
                $contador += 1;
                $contreg  += 1;
                $emiteremessa->geraTransacao($caminho, $nmarquivo,$nrorg, $cdempresa, $boleto['CDCARTCOBBOL'], $boleto['CDAGENCIA'], $boleto['CDCONTCORR'],
                    $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['VRPEMORBREC'], $boleto['IDIMPRBOLBOL'], $boleto['CDNOSNUMREC'], $boleto['DTVENBOLREC'],
                    $boleto['CDINS1COBBOL'], $boleto['QTDIAPROTBOL'], $boleto['VRMORDIABREC'], $boleto['VRFINALBREC'], $idoperacao,
                    $boleto['VRPEMULBREC'], $boleto['IDTPIJURCLIE'], $boleto['NRINSJURCLIE'], $boleto['CDCLIENTE'], $boleto['CDINS2COBBOL'], $contreg, $cnabrec->getIdtipcnabrec(),$cnabrec->getCdFilial(),
                    $boleto['CDDIAPROTBOL'], $contador, $cdreducont,$cdbanco
                );
            }
            $contlote  = 1;
            $contreg  += 1;
            $emiteremessa->GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote, $contreg, $cdreducont);
        }
    }

    /**
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $caminho
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $idoperacao
     * @param $cdreducont
     * @param $nmarquivo
     * @param EntityManager $entitymanager
     * @param TitulosAReceberFactory $titulosareceberfactory
     */
    public static function geraCnabBrasil(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
        $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
        EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){
        
        $emiteremessa = GeraCnabService::factory($cnabrec, $entitymanager, $titulosareceberfactory);
        
        if(count($boletos) != 0){
            $contreg  = 3;
            $contador = 1;
            
            $emiteremessa->geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, $boletos[(count($boletos)-1)]['CDCARTCOBBOL'], null, $contreg, $cnabrec->getCdFilial(), $contador);
            $emiteremessa->geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, $boletos[(count($boletos)-1)]['CDCARTCOBBOL'], $cnabrec->getCdFilial());
            foreach ($boletos as $boleto) {

                $emiteremessa->geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(),
                    $nrcnabrec, $idoperacao, $boleto['CDAGENCIA'], $cdcontacorrente, $cdcontcorrcobranca, $boleto['CDNOSNUMREC'],
                    $boleto['IDCARTCOBREC'], $boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $boleto['DTVENBOLREC'], $boleto['VRFINALBREC'], $boleto['VRPEMORBREC'], $boleto['VRMORDIABREC'],
                    $boleto['VRPEMULBREC'], $boleto['VRMULTABREC'], $boleto['VRFINALBREC'], $boleto['CDCARTCOBBOL'], $boleto['CDDIAPROTBOL'], $cdreducont,  $boleto['IDIMPRBOLBOL'],
                    $boleto['QTDIAPROTBOL'], $contador
                );
                $contador += 1;
                
                $emiteremessa->geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $boleto['CDCLIENTE'], $idoperacao, $cdempresa, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $contador);
                $contador += 1;
                /** @var Boletreceber $boletreceber */
                $boletreceber = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg(
                    $boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $boleto['NRORG']
                );
                
                if(trim($boletreceber->getTxmensbolbol()) or $boleto['VRPEMULBREC'] != 0 or $boleto['VRMULTABREC'] != 0){
                    $emiteremessa->geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $boleto['DTVENBOLREC'], $boleto['VRPEMULBREC'], $boleto['NRLANBOLREC'], $boleto['VRMULTABREC'], $contador);
                    $contador += 1;
                }
                $contreg  += 3;
            }
            $emiteremessa->GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco, $contreg);
            $contlote  = 1;
            $contreg  += 1;
            $emiteremessa->GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote, $contreg, $cdreducont);
        }
    }

    /**
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $caminho
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $idoperacao
     * @param $cdreducont
     * @param $nmarquivo
     * @param EntityManager $entitymanager
     * @param TitulosAReceberFactory $titulosareceberfactory
     */
    public static function geraCnabCaixa(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
        $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
        EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){
        $emiteremessa = GeraCnabService::factory($cnabrec, $entitymanager, $titulosareceberfactory);

        if(count($boletos) != 0){
            $contreg  = 3;
            $contador = 1;
            
            $emiteremessa->geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, null, null, $contreg, $cnabrec->getCdFilial(), $contador);
            $emiteremessa->geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont,null, $cnabrec->getCdFilial());
            foreach ($boletos as $boleto) {

                $emiteremessa->geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(),
                    $nrcnabrec, $idoperacao, $boleto['CDAGENCIA'], $cdcontacorrente, $cdcontcorrcobranca, $boleto['CDNOSNUMREC'],
                    $boleto['IDCARTCOBREC'], $boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $boleto['DTVENBOLREC'], $boleto['VRFINALBREC'], $boleto['VRPEMORBREC'], $boleto['VRMORDIABREC'],
                    $boleto['VRPEMULBREC'], $boleto['VRMULTABREC'], $boleto['VRFINALBREC'], $boleto['CDCARTCOBBOL'], $boleto['CDDIAPROTBOL'], $cdreducont,  $boleto['IDIMPRBOLBOL'],
                    $boleto['QTDIAPROTBOL'], $contador
                );
                $emiteremessa->geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $boleto['CDCLIENTE'], $idoperacao, $cdempresa, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $contador);
                $emiteremessa->geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $boleto['DTVENBOLREC'], $boleto['VRPEMULBREC'], $boleto['NRLANBOLREC'], $boleto['VRMULTABREC'], $contador);

                $contador += 1;
                $contreg  += 3;
            }
            $emiteremessa->GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco, $contreg);
            $contlote  = 1;
            $contreg  += 1;
            $emiteremessa->GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote, $contreg, $cdreducont);
        }
    }

    /**
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $caminho
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $idoperacao
     * @param $cdreducont
     * @param $nmarquivo
     * @param EntityManager $entitymanager
     * @param TitulosAReceberFactory $titulosareceberfactory
     */
    public static function geraCnabSofisa(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec,
        $idoperacao, $cdreducont, $nmarquivo,
        EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){
        $emiteremessa = GeraCnabService::factory($cnabrec, $entitymanager, $titulosareceberfactory);
        
        if(count($boletos) != 0){
            $contreg  = 1;
            $contador = 1;
            $emiteremessa->geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, null, null, $contreg, $cnabrec->getCdFilial(), $contador);
            foreach($boletos as $boleto){
                $contador += 1;
                $contreg  += 1;
                
                $emiteremessa->geraTransacao($caminho, $nmarquivo, $cdempresa,$nrorg, $boleto['CDCARTCOBBOL'], $boleto['CDAGENCIA'], $boleto['CDCONTCORR'],
                    $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['VRPEMORBREC'], $boleto['IDIMPRBOLBOL'], $boleto['CDNOSNUMREC'], $boleto['DTVENBOLREC'],
                    $boleto['CDINS1COBBOL'], $boleto['QTDIAPROTBOL'], $boleto['VRMORDIABREC'], $boleto['VRFINALBREC'], $idoperacao,
                    $boleto['VRPEMULBREC'], $boleto['IDTPIJURCLIE'], $boleto['NRINSJURCLIE'], $boleto['CDCLIENTE'], $boleto['CDINS2COBBOL'], $contreg, $cnabrec->getIdtipcnabrec(), $cnabrec->getCdFilial(),null, null,$cdreducont,$cdbanco
                );
            }
            $contlote  = 1;
            $contreg  += 1;
            $emiteremessa->GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote, $contreg, $cdreducont);
        }
    }
    
     /**
     * Gera Cnab Votorantim
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $caminho
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrcnabrec
     * @param $idoperacao
     * @param $cdcontacorrente
     * @param $cdcontcorrcobranca
     * @param $cdreducont
     * @param $nrcnabrec
     * @param $nmarquivo
     */
    public static function geraCnabVotorantim(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
        $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
        EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){
        $emiteremessa = GeraCnabService::factory($cnabrec, $entitymanager, $titulosareceberfactory);
        if (count($boletos) != 0){
            $contreg  = 3;
            $contador = 1;
            $emiteremessa->geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, null, null, $contreg, $cnabrec->getCdFilial(), $contador);
            $emiteremessa->geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont,null,$cnabrec->getCdFilial());
            foreach ($boletos as $boleto) {

                $emiteremessa->geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(),
                    $nrcnabrec, $idoperacao, $boleto['CDAGENCIA'], $cdcontacorrente, $cdcontcorrcobranca, $boleto['CDNOSNUMREC'],
                    $boleto['IDCARTCOBREC'], $boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $boleto['DTVENBOLREC'], $boleto['VRFINALBREC'], $boleto['VRPEMORBREC'], $boleto['VRMORDIABREC'],
                    $boleto['VRPEMULBREC'], $boleto['VRMULTABREC'], $boleto['VRFINALBREC'], $boleto['CDCARTCOBBOL'], $boleto['CDDIAPROTBOL'], $cdreducont,  $boleto['IDIMPRBOLBOL'],
                    $boleto['QTDIAPROTBOL'], $contador
                );
                $contador += 1;
                $emiteremessa->geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $boleto['CDCLIENTE'], $idoperacao, $cdempresa, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $contador);
                $contador += 1;
                $emiteremessa->geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $boleto['DTVENBOLREC'], $boleto['VRPEMULBREC'], $boleto['NRLANBOLREC'], $boleto['VRMULTABREC'], $contador);

                $contador += 1;
                $contreg  += 3;
            }
            $emiteremessa->GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco, $contreg);
            $contlote  = 1;
            $contreg  += 1;
            $emiteremessa->GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote, $contreg, $cdreducont);
        }
    }
    
    /**
     * Gera Cnab Daycoval
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $caminho
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $idoperacao
     * @param $cdreducont
     * @param $nmarquivo
     * @param EntityManager $entitymanager
     * @param TitulosAReceberFactory $titulosareceberfactory
     */
    public static function geraCnabDaycoval(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec,
        $idoperacao, $cdreducont, $nmarquivo,
        EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){

        $emiteremessa = GeraCnabService::factory($cnabrec, $entitymanager, $titulosareceberfactory);
        
        if(count($boletos) != 0){
            $contreg  = 1;
            $contador = 1;
            $emiteremessa->geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, null, null, $contreg, $cnabrec->getCdFilial(), $contador);
            foreach($boletos as $boleto){
                $contador += 1;
                $contreg  += 1;
                
                $emiteremessa->geraTransacao($caminho, $nmarquivo,$nrorg, $cdempresa, $boleto['CDCARTCOBBOL'], $boleto['CDAGENCIA'], $boleto['CDCONTCORR'],
                    $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $boleto['VRPEMORBREC'], $boleto['IDIMPRBOLBOL'], $boleto['CDNOSNUMREC'], $boleto['DTVENBOLREC'],
                    $boleto['CDINS1COBBOL'], $boleto['QTDIAPROTBOL'], $boleto['VRMORDIABREC'], $boleto['VRFINALBREC'], $idoperacao,
                    $boleto['VRPEMULBREC'], $boleto['IDTPIJURCLIE'], $boleto['NRINSJURCLIE'], $boleto['CDCLIENTE'], $boleto['CDINS2COBBOL'], $contreg, $cnabrec->getIdtipcnabrec(), $cnabrec->getCdFilial(),$boleto['CDDIAPROTBOL'], $contador, $cdreducont,$cdbanco
                );
                $contreg  += 1;
                $emiteremessa->geraRegistroMensagem($caminho, $nmarquivo,$boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $contreg);
            }
            $contlote  = 1;
            $contreg  += 1;
            $emiteremessa->GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote, $contreg, $cdreducont);
        }
    }
    
	/**
     * Gera Cnab Banrisul
     * @param $boletos
     * @param Cnabrec $cnabrec
     * @param $caminho
     * @param $nrorg
     * @param $cdbanco
     * @param $cdempresa
     * @param $cdidrec
     * @param $nrcnabrec
     * @param $idoperacao
     * @param $cdcontacorrente
     * @param $cdcontcorrcobranca
     * @param $cdreducont
     * @param $nrcnabrec
     * @param $nmarquivo
     */
     
    public static function geraCnabBanrisul(
        $boletos, Cnabrec $cnabrec, $caminho, $nrorg, $cdbanco, $cdempresa, $cdidrec, $nrcnabrec,
        $idoperacao, $cdcontacorrente, $cdcontcorrcobranca, $cdreducont, $nrcnabrec1, $nmarquivo,
        EntityManager $entitymanager, TitulosAReceberFactory $titulosareceberfactory
    ){
        $emiteremessa = GeraCnabService::factory($cnabrec, $entitymanager, $titulosareceberfactory);
        if (count($boletos) != 0){
            $contreg  = 3;
            $contador = 1;
            $emiteremessa->geraHeaderArquivo($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont, null, null, $contreg, $cnabrec->getCdFilial(), $contador);
            $emiteremessa->geraHeaderLote($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(), $cnabrec->getNrcnabrec(), $cdreducont,null,$cnabrec->getCdFilial());
            foreach ($boletos as $boleto) {

                $emiteremessa->geraDetalheSegmentoP($caminho, $nmarquivo, $nrorg, $cdbanco, $cdempresa, $cdidrec, $cnabrec->getNrseqremessarec(),
                    $nrcnabrec, $idoperacao, $boleto['CDAGENCIA'], $cdcontacorrente, $cdcontcorrcobranca, $boleto['CDNOSNUMREC'],
                    $boleto['IDCARTCOBREC'], $boleto['NRLANBOLREC'], $boleto['CDFILIAL'], $boleto['DTVENBOLREC'], $boleto['VRFINALBREC'], $boleto['VRPEMORBREC'], $boleto['VRMORDIABREC'],
                    $boleto['VRPEMULBREC'], $boleto['VRMULTABREC'], $boleto['VRFINALBREC'], $boleto['CDCARTCOBBOL'], $boleto['CDDIAPROTBOL'], $cdreducont,  $boleto['IDIMPRBOLBOL'],
                    $boleto['QTDIAPROTBOL'], $contador
                );
                $contador += 1;
                $emiteremessa->geraDetalheSegmentoQ($caminho, $nmarquivo, $nrorg, $cdbanco, $boleto['CDCLIENTE'], $idoperacao, $cdempresa, $boleto['CDFILIAL'], $boleto['NRLANBOLREC'], $contador);
                $contador += 1;
                $emiteremessa->geraDetalheSegmentoR($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $boleto['DTVENBOLREC'], $boleto['VRPEMULBREC'], $boleto['NRLANBOLREC'], $boleto['VRMULTABREC'], $contador);
                
                // $contador += 1; TODO(Impementar após adicionar os campos na entities e passar os campos na tela de conta corrente)
                // $emiteremessa->geraDetalheSegmentoY01($caminho, $nmarquivo, $nrorg, $cdbanco, $idoperacao, $boleto['DTVENBOLREC'], $boleto['VRPEMULBREC'], $boleto['NRLANBOLREC'], $boleto['VRMULTABREC'], $contador,$cdreducont);
                
                $contador += 1;
                $contreg  += 3;
            }
            $emiteremessa->GeraTraillerLote($caminho, $nmarquivo, $nrorg, $cdbanco, $contreg);
            $contlote  = 1;
            $contreg  += 1;
            $emiteremessa->GeraTrailler($caminho, $nmarquivo, $nrorg, $cdbanco, $contlote, $contreg, $cdreducont);
        }
    }
    
    /**
     * @param $dsarqcnabrec
     * @param $nmarqcnabrec
     * @return array
     */
    public function createFileFieldValue($dsarqcnabrec, $nmarqcnabrec)
    {
        // Local do Arquivo CNAB Remessa Receber
        $local_arquivo = $dsarqcnabrec.$nmarqcnabrec;
        $date = new \DateTime();
        $data = file_get_contents($local_arquivo);
        $base64encoded = base64_encode($data);
        return array(
            array(
                'b64File' => 'data:;base64,'.$base64encoded,
                'lastModified' => $date->getTimestamp(),
                'lastModifiedDate' => $date->format('Y-d-m H:i:s'),
                'name' => $nmarqcnabrec,
                'size' => count($data),
                'type' => ""
            )
        );
    }

    /**
     * @param $dsarqcnabrec
     * @param $nmarqcnabrec
     * @return array
     */
    public function uploadArqRemessaReceber(
        $dsarqcnabrec, $nmarqcnabrec
    ){

        // Prepara Array de Upload Arquivo
        $row = array(
            self::FILE_FIELD_NAME => $this->createFileFieldValue($dsarqcnabrec, $nmarqcnabrec)
        );

        
        /*$fileContent = file_get_contents($tmpFile);
        $fileBase64 = base64_encode($fileContent);
    
        $fileBase64Zeedhi = 'data: '.mime_content_type($tmpFile).';base64,'.$fileBase64;
        
        $row = [
            'FILE' => [
                [
                    "name" => time().'.xml', 
                    "type" => 'application/xml', 
                    "size" => strlen($fileContent), 
                    "path" => mime_content_type($tmpFile), 
                    "b64File" => $fileBase64Zeedhi
                ]
            ],
            'FOLDER' => 'REINF'
        return $this->fileUploader->uploadFile($row)[0]->getRows()[0]['FILE'][0]['PATH'];
        ];*/
        
        // Upload Arquivo
        $this->server->uploadFilesInRow($row, self::FILE_FIELD_NAME, self::CUSTOM_FOLDER);

        return $row[self::FILE_FIELD_NAME][0]['path'];
    }


    /**
     * Formata Data Hora
     * @param $data
     * @return \DateTime|null|\Setforms2\Shared\ORM\DateTime|string
     */
    public static function formataDataHora($data)
    {
        if (!$data)
            return null;

        if (strlen($data) == 19)
            $data = DateUtil::truncateString($data);
        $data = DateUtil::getDataDeString($data);

        return $data;
    }
}