<?php

namespace Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Banco;

use Financeiro\Entities\Boletreceber;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\LeitorRetorno;
use Teknisa\FIN\Modules\ContaReceber\Services\Retorno\Retorno;

class LeitorRetornoBanrisul extends LeitorRetorno{

    CONST ID_TP_ARQUIVO_HEADER_ARQUIVO = '0';
    CONST ID_TP_ARQUIVO_HEADER_LOTE = '1';
    CONST ID_TP_ARQUIVO_DETALHE = '3';
    CONST ID_TP_TRAILER_LOTE = '5';
    CONST ID_TP_TRAILER_ARQUIVO = '9';

    private $arquivo;
    /** @var $retorno Retorno */
    private $retorno;

    /**
     * @param $arquivo
     * @param $cdempresa
     * @param $cdbanco
     * @throws \Exception
     */
    public function __construct($arquivo, $cdempresa, $cdbanco, $nrorg){
        try {
            $this->cdbanco = $cdbanco;
            $this->arquivo = $arquivo;
            $this->cdempresa = $cdempresa;
            $this->nrorg = $nrorg;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getEstruturaArquivo(){
        return array(
            self::ID_TP_ARQUIVO_HEADER_ARQUIVO,
            self::ID_TP_ARQUIVO_HEADER_LOTE,
            self::ID_TP_ARQUIVO_DETALHE,
            self::ID_TP_TRAILER_LOTE,
            self::ID_TP_TRAILER_ARQUIVO
        );
    }

    public function leArquivo(){
        try {
            foreach ($this->arquivo as $linha) {
                $cont = $this->contlinha++;
                if ($linha){
                    $idsegmento = substr($linha, 13, 1);
                    if($idsegmento == 'U')
                        $cont = $cont - 1;
                    else
                        $this->retorno = new Retorno();

                    // Busca o Tipo de Registro do Arquivo
                    $idRegistro = substr($linha, 7, 1);
                    if(self::validaArquivo($linha, self::ID_TP_ARQUIVO_HEADER_ARQUIVO, self::getEstruturaArquivo(), $this->contlinha)) {
                        switch ($idRegistro) {

                            case(self::ID_TP_ARQUIVO_HEADER_ARQUIVO):
                                self::leHeaderArquivo($linha);
                                break;

                            case(self::ID_TP_ARQUIVO_HEADER_LOTE):
                                self::leHeaderLote($linha);
                                break;

                            case(self::ID_TP_ARQUIVO_DETALHE):
                                self::leDetalhe($linha);
                                break;

                            case(self::ID_TP_TRAILER_LOTE):
                                break;

                            case(self::ID_TP_TRAILER_ARQUIVO):
                                break;

                            default:
                                throw new \Exception('Arquivo inválido.');
                        }
                    }
                    $this->retornos[$cont] = $this->retorno;
                }
            }
            self::preparaOcorrencias($this->cdbanco, $this->nrorg);

        } catch (\Exception $e) {
            throw $e;
        }
    }

    //########################################################################
    //############# Identificação do Registro Header #########################
    //########################################################################
    private function leHeaderArquivo($header){

        // CAMPO: Código do Banco na Compensação
        // POSIÇÃO: 1 a 3
        // PICTURE: 9(3)
        if(substr($header, 0, 3) != self::CODIGO_COMPENSACAO_BANRISUL)
            self::gravaErros($this->contlinha, 'Banco Inválido.', true);

        // CAMPO: Código Remessa / Retorno
        // POSIÇÃO: 143 a 143
        // PICTURE: 9(1)
        //'1' = Remessa (Cliente > Banco)
        //'2' = Retorno (Banco > Cliente)
        if (substr($header, 142, 1) != '2'){
            self::gravaErros($this->contlinha, 'Arquivo inválido.', true);
        }
    }

    //########################################################################
    //############# Identificação do Registro Header de Lote #################
    //########################################################################
    private function leHeaderLote($header){

        // CAMPO: Tipo de Inscrição da empresa
        // POSIÇÃO: 018 a 018
        // PICTURE: 9(1)
        $tpinscricao = trim(substr($header, 17, 1));
        if (empty($tpinscricao)){
            self::gravaErros($this->contlinha, 'Tipo de inscrição da empresa inválido.', true);
        }
    

        // CAMPO: Nº de Inscrição da Empresa
        // POSIÇÃO: 019 a 033
        // PICTURE: 9(15)
        $inscricao = trim(substr($header, 18, 15));
        if (empty($inscricao)){
            self::gravaErros($this->contlinha, 'Inscrição da empresa inválida.', true);
        }
    }

    //########################################################################
    //############# Identificação do Registro Detalhe ########################
    //########################################################################
    private function leDetalhe($detalhe){
        
        // CAMPO: Código de Movimento Retorno
        // POSIÇÃO: 16 a 17
        // PICTURE: 9(2)
        $validaOcorre = substr($detalhe, 15, 2);
        
        //############# Identificação do Registro de Segmento T
        
        // CAMPO: Código Segmento do Registro Detalhe
        // POSIÇÃO: 14 a 14
        // PICTURE: X(1)
        // "T" ou "U"
        
        if(substr($detalhe, 13, 1) == 'T'){
            
            // CAMPO: Agencia
            // POSIÇÃO: 18 a 22
            // PICTURE: 9(5)
            $agencia = trim(substr($detalhe, 18,5));
            
            // CAMPO: Conta
            // POSIÇÃO: 24 a 35
            // PICTURE: 9(12)
            $conta = trim(substr($detalhe, 24, 12));
            
            // CAMPO:Identificação do Título na Empresa
            // POSIÇÃO: 106 a 130
            // PICTURE: 9(25)
            // FILIAL(4 posições) + NºLancamentoBoleto(10 Posições)
            $cdfilial = substr($detalhe, 105, 4);
            $nrlanbolrec = substr($detalhe, 109, 10);
            
            // CAMPO: Identificação do Título(Nosso Numero)
            // POSIÇÃO: 38 a 57
            // PICTURE: 9(10)
            $idtitulobanco = trim(substr($detalhe, 37, 10));
            
            // CAMPO: Código da Carteira
            // POSIÇÃO: 58 a 58
            // PICTURE: 9(1)
            $cdcartbol = trim(substr($detalhe,57 , 1));
            
            $cdempresa = $this->cdempresa;
            
            $nossonumero = Boletreceber::getRepository();
            
            // /** @var $nossonumero \Financeiro\Repositories\Boletreceber */
            // $cdnosnumrec = $nossonumero->getNossoNumero($idtitulobanco, $cdempresa);
            $boleto = false;
            if(!empty($cdfilial) && !empty($nrlanbolrec))
            /** @var $boleto Boletreceber */
            $boleto = Boletreceber::findOneByNrlanbolrecAndCdfilialAndNrorg($nrlanbolrec, $cdfilial, $this->nrorg);
            $this->retorno->setBoleto('BOLETO');
            $this->retorno->setCdlinha(str_pad($this->contlinha, 4, 0, STR_PAD_LEFT));
            $this->retorno->setCdfilial($cdfilial);
            $this->retorno->setNrlanbolrec($nrlanbolrec);
            $this->retorno->setNossonumero($idtitulobanco);
            $this->retorno->setCarteira($cdcartbol);
            $this->retorno->setCdagencia($agencia);
            $this->retorno->setCdconta($conta);

            if($boleto) {
                $this->retorno->setVrfinalbrec($boleto->getVrfinalbrec());
                $this->retorno->setVrbolrec($boleto->getVrbolrec());
                $this->retorno->setIdstatrec($boleto->getIdstatbrec());
                $this->retorno->setCdfilial($boleto->getCdfilial());
                $this->retorno->setNrlanbolrec($boleto->getNrlanbolrec());
                $this->retorno->setCdtpbaixarec($boleto->getCdtpbaixarec());
                $this->retorno->setCdreducont($boleto->getCdreducont());
            }

            // CAMPO: Identificação da Ocorrência no Banco
            // POSIÇÃO: 016 a 017
            // PICTURE: 9(2)
            if (empty($validaOcorre))
                $this->gravaErros($this->contlinha, 'Ocorrência inválida.');
            else
                $this->retorno->setIdsitucnabr(self::situacaoCnab($validaOcorre));

            // CAMPO: Data de Vencimento do título
            // POSIÇÃO: 074 a 081
            // PICTURE:
            $vencimento = substr($detalhe, 72, 8);
            $vencimento = substr($vencimento, 0, 2) . "/" . substr($vencimento, 2, 2) . "/" . substr($vencimento, -4);
            $this->retorno->setDtvencimento($vencimento);

            // CAMPO: Valor nominal do título
            // POSIÇÃO: 082 a 096
            // PICTURE: 9(13)V9(2)
            $this->retorno->setVrnominal(round(floatval(substr($detalhe, 81, 15)) / 100, 2));

            // CAMPO: Erros - Registros rejeitados ou alegação do sacado
            // POSIÇÃO: 214 a 223
            // PICTURE: X(10)
            $iderros = trim(substr($detalhe, 213, 10));
            if(self::situacaoCnab($validaOcorre) == '01'){
                $primeiroPar = substr($iderros, 0);
                $this->retorno->setOcorrencia($primeiroPar);
                    
            }
        }
        else{
            // CAMPO: Desconto concedido
            // POSIÇÃO: 033 a 047
            // PICTURE: 9(13)V9(2)

            // CAMPO: Abatimento concedito sobre o título
            // POSIÇÃO: 048 a 062
            // PICTURE: 9(13)V9(2)

            // CAMPO: IOF Recolhido
            // POSIÇÃO: 063 a 077
            // PICTURE: 9(13)V9(2)

            // CAMPO: Valor de Outras Despesas
            // POSIÇÃO: 108 a 122
            // PICTURE: 9(13)V9(2)
            $this->retorno->setVrdesconto(
                round(floatval(substr($detalhe, 32, 15)) / 100, 2) +
                round(floatval(substr($detalhe, 47, 15)) / 100, 2) +
                round(floatval(substr($detalhe, 62, 15)) / 100, 2) +
                round(floatval(substr($detalhe, 107, 15)) / 100, 2)
            );

            // CAMPO: Juros/Multa/Encargos
            // POSIÇÃO: 018 a 032
            // PICTURE: 9(13)V9(2)

            // CAMPO: Valor de outros Creditos
            // POSIÇÃO: 123 a 137
            // PICTURE: 9(13)V9(2)
            $this->retorno->setVracrescimo(
                round(floatval(substr($detalhe, 17, 15)) / 100, 2) +
                round(floatval(substr($detalhe, 122, 15)) / 100, 2)
            );

            // CAMPO: Valor Pago pelo Sacado
            // POSIÇÃO: 078 a 092
            // PICTURE: 9(13)V9(2)
            $this->retorno->setVrpago(
                round(floatval(substr($detalhe, 77, 15)) / 100, 2)
            );
            
            $situacaoCnab = self::situacaoCnab($validaOcorre);
            if($situacaoCnab && $situacaoCnab == '09'){
                
                // CAMPO: Data de efetivação do crédito
                // POSIÇÃO: 146 a 153
                // PICTURE: X(8)
                $pagamento = substr($detalhe, 145, 8);
                $this->retorno->setDtpagamento($pagamento);
            }
        }
    }

    /**
     * @param $cdocorrencia
     * @return string
     */
    private function situacaoCnab($cdocorrencia){
        
        if($cdocorrencia == '03' or $cdocorrencia == '26' or $cdocorrencia == '30' or $cdocorrencia == '37'){
           return '01'; //Retorno com Erro 
        } else if($cdocorrencia == '02'){
             return '20';// Agendado
        } elseif($cdocorrencia == '09' or $cdocorrencia == '17' or $cdocorrencia == '25' ){
            return '90'; //Baixado
        } else {
             return '10'; //Sem retorno
        }
    }
}