<?php

namespace Teknisa\FIN\Services;

use Teknisa\Libs\Util\DateUtil;
use Doctrine\ORM\EntityManager;
use Teknisa\FIN\Factories\AutomacaoFinanceiraFactory;
use Teknisa\FIN\Modules\ContaReceber\Services\TitulosAReceberService;

class AutomacaoFinanceiraService {
    
    /**
     * @param Environment                   $environment
     * @param TitulosAReceberService        $titulosAReceberService
     * @param AutomacaoFinanceiraFactory    $factory
     */
    public function __construct(EntityManager $entityManager, TitulosAReceberService $titulosAReceberService, AutomacaoFinanceiraFactory $factory) {
        $this->entityManager = $entityManager;
        $this->titulosAReceberService = $titulosAReceberService;
        $this->factory       = $factory;
    }
    
    /**
     * Executa Envio de Boleto Emitido
     */
     public function executeEmissao($nrorg, $useVpd, $newEntityManager, $newFileUploader, $request, $response) {
	    //Conectar ao Banco de Dados
	    $this->factory->conectar($nrorg,$useVpd, $newEntityManager,$newFileUploader, $request, $response);
	    
        //Executa Envio de Boleto Emitido
        $apuracao = $this->execEnvioBoletoEmitido($nrorg, $request, $response);
    }
        
    /**
     * Executa Envio de Boleto Emitido
     */
    public function execEnvioBoletoEmitido($nrorg, $request, $response) {
        // Busca Parametrização Geral
        $geralrec = $this->factory->getGeralrec($nrorg);
        
        // Verifica existência de Parametrização Geral
        if($geralrec){
            // Verifica utilização do Envio de Boleto Automático
            if($geralrec->getIdutenvbolauto() == 'S'){
                $qtdiaantenvbol  = $geralrec->getQtdiaantenvbol();
                
                $dataatual  = DateUtil::getDataAtual();
                $vencimento = DateUtil::adicionaIntervalo($dataatual, $qtdiaantenvbol)->format('d/m/Y');
                
                // Busca Boletos Habilitados para Envio
                $boletrecebers = $this->factory->getBoletosParaEnvioEmissao($nrorg, $vencimento);
                
                // Verifica existência de Boletos para Envio
                if($boletrecebers){
                    $assunto  = $geralrec->getDsassmailenvbol();
                    $mensagem = $geralrec->getDsmsgmailenvbol();
                    
                    // Envia Boleto Emitido por E-Mail
                    $this->titulosAReceberService->enviaTituloPorEmail(null, $assunto, $mensagem, $boletrecebers, $request, $response, 'SB');
                }else var_dump('Atenção. Não há títulos a serem enviados.');
            }else var_dump('Atenção. Organização não utiliza envio de título automático.');
        }else var_dump('Operação bloqueada. Parâmetros de títulos a receber não encontrada para organização.');
    }
    
    /**
     * Executa Envio de Carta de Cobrança
     */
     public function executeCobranca($nrorg, $useVpd, $newEntityManager, $newFileUploader, $request, $response) {

	    //Conectar ao Banco de Dados
	    $this->factory->conectar($nrorg,$useVpd, $newEntityManager,$newFileUploader, $request, $response);
	    
        //Executa Envio de Boleto Emitido
        $apuracao = $this->execEnvioCartaCobranca($nrorg, $request, $response);
    }
    
    /**
     * Executa Envio de Carta de Cobrança
     */
    public function execEnvioCartaCobranca($nrorg, $request, $response) {
        // Busca Parametrização Geral
        $geralrec = $this->factory->getGeralrec($nrorg);
        
        // Verifica existência de Parametrização Geral
        if($geralrec){
            // Verifica utilização do Envio de Carta de Cobrança Automática
            if($geralrec->getIdutenvcartauto() == 'S'){
                $qtdiaposenvcar  = $geralrec->getQtdiaposenvcar();
                
                $dataatual  = DateUtil::getDataAtual();
                $vencimento = DateUtil::subtraiIntervalo($dataatual, $qtdiaposenvcar)->format('d/m/Y');
                
                // Busca Boletos Habilitados para Envio
                $boletrecebers = $this->factory->getBoletosParaEnvioCobranca($nrorg, $vencimento);
                
                // Verifica existência de Boletos para Envio
                if($boletrecebers){
                    $assunto  = $geralrec->getDsassmailenvcar();
                    $mensagem = $geralrec->getDsmsgmailenvcar();
                    
                    // Envia Carta de Cobrança por E-Mail
                    $this->titulosAReceberService->enviaTituloPorEmail(null, $assunto, $mensagem, $boletrecebers, $request, $response, 'SC');
                }else var_dump('Atenção. Não há títulos a serem enviados.');
            }else var_dump('Atenção. Organização não utiliza envio de carta de cobrança automática.');
        }else var_dump('Operação bloqueada. Parâmetros de títulos a receber não encontrada para organização.');
    }
    
}