/******************************************************************************/
/* Sistemas Integrados TECNISA                                                */
/*                                                                            */
/* Compras e Suprimentos                                                      */
/*                                                                            */
/******************************************************************************/
/*                                                                            */
/* Nome: HISTORICO_COTACAO                                                          */
/*                                                                            */
/* Descrio:                                                                 */
/* Retorna dados para o histrico de cotaes                                                  */
/*                                                                            */
/*                                                                            */
/* Criao:                                                                   */
/* 14/04/2000 - Adriano de Arajo                                            */
/*                                                                            */
/* Manutenes:                                                               */
/*   /  /   -                                                                 */
/*                                                                            */
/*                                                                            */
/******************************************************************************/

/******************************************************************************/
/* PARAMETROS:                                                                */
/*                                                                            */
/******************************************************************************/


IF EXISTS (SELECT * FROM sysobjects O, sysusers U
          WHERE 
            o.name = 'HISTORICO_COTACAO'  and
            type = 'P'                  and
            u.uid = o.uid               and
            u.name = USER)

   DROP PROCEDURE HISTORICO_COTACAO;
GO


CREATE PROCEDURE HISTORICO_COTACAO @P_CDFILIAL     VARCHAR(4),
                                   @P_NRCOLEPRECO  VARCHAR(10),
                                   @P_CDPRODUTO    VARCHAR(10),
                                   @P_CDMARCA      VARCHAR(5),
                                   @P_CDFORNECED   VARCHAR(21),
                                   @P_CDREGIAO     VARCHAR(3),
                                   @P_VRULTCOT_PRUN  NUMERIC(15,5) OUTPUT,
                                   @P_VRULTCOT_ECO   NUMERIC(15,5) OUTPUT,
                                   @P_NMFORN_NORM    VARCHAR(10) OUTPUT,
                                   @P_VRFNORM_PRUN   NUMERIC(15,5) OUTPUT,
                                   @P_VRFNORM_ECO    NUMERIC(15,5) OUTPUT,
                                   @P_NMFORN_ALTER    VARCHAR(10) OUTPUT,
                                   @P_VRFALTER_PRUN   NUMERIC(15,5) OUTPUT,
                                   @P_VRFALTER_ECO    NUMERIC(15,5) OUTPUT,
                                   @P_PERVARIACAO     NUMERIC(5,2)  OUTPUT,
                                   @P_DTINIVAL_ULTCOT DATETIME   OUTPUT,
                                   @P_DTFINVAL_ULTCOT DATETIME   OUTPUT,
                                   @P_DTINIVAL_MINPRFN DATETIME  OUTPUT,
                                   @P_DTFINVAL_MINPRFN DATETIME  OUTPUT,
                                   @P_DTINIVAL_MINPRFA DATETIME  OUTPUT,
                                   @P_DTFINVAL_MINPRFA DATETIME  OUTPUT
AS
DECLARE @V_ATUAL_PR  NUMERIC(15,5);
DECLARE @V_DTINIVALITM    DATETIME;
DECLARE @V_DTFINVALITM    DATETIME;

BEGIN
  SELECT
     @V_DTINIVALITM = ITEMCOLE.DTINIVALITM,
     @V_DTFINVALITM = ITEMCOLE.DTFINVALITM,
     @V_ATUAL_PR = ITEMCOLE.VRPRECUNIITM
  FROM
     ITEMCOLE
  WHERE
     ITEMCOLE.CDFILIAL    = @P_CDFILIAL    AND
     ITEMCOLE.NRCOLEPRECO = @P_NRCOLEPRECO AND
     ITEMCOLE.CDREGIAO    = @P_CDREGIAO    AND
     ITEMCOLE.CDFORNECED  = @P_CDFORNECED  AND
     ITEMCOLE.CDPRODUTO   = @P_CDPRODUTO   AND
     ITEMCOLE.CDMARCA     = @P_CDMARCA

   /*  ltima Cotao do Fornecedor.*/
   SELECT
     @P_VRULTCOT_PRUN = ITEMCOLE.VRPRECUNIITM,
     @P_VRULTCOT_ECO  = ITEMCOLE.VRPRECECOITM,
     @P_DTINIVAL_ULTCOT  = ITEMCOLE.DTINIVALITM,
     @P_DTFINVAL_ULTCOT  = ITEMCOLE.DTFINVALITM
   FROM
     ITEMCOLE
   WHERE
     ITEMCOLE.CDFILIAL   = @P_CDFILIAL    AND
     ITEMCOLE.CDPRODUTO  = @P_CDPRODUTO   AND
     ITEMCOLE.CDFORNECED = @P_CDFORNECED  AND
     ITEMCOLE.CDREGIAO   = @P_CDREGIAO    AND
     ITEMCOLE.NRCOLEPRECO=
       /* Selecionando a cotacao de maior nmero, a ltima */
       (SELECT
            MAX(ITEMCOLE.NRCOLEPRECO) AS COLETA
         FROM
            ITEMCOLE
         WHERE
             ITEMCOLE.CDFILIAL    =  @P_CDFILIAL    AND
             ITEMCOLE.NRCOLEPRECO <> @P_NRCOLEPRECO AND
             ITEMCOLE.CDPRODUTO   =  @P_CDPRODUTO   AND
             ITEMCOLE.CDREGIAO    =  @P_CDREGIAO    AND
             ITEMCOLE.CDFORNECED  =  @P_CDFORNECED)

   /* Menor preo econmico entre os fornecedores normais */
   SELECT
     @P_NMFORN_NORM  = FORNECEDOR.NMFANTFORN,
     @P_VRFNORM_PRUN = ITEMCOLE.VRPRECUNIITM,
     @P_VRFNORM_ECO  = ITEMCOLE.VRPRECECOITM

   FROM FORNECEDOR,
        ITEMCOLE,
        PARASUPRI
   WHERE
      PARASUPRI.CDFILIAL  = @P_CDFILIAL   AND

      ITEMCOLE.DTINIVALITM >= @V_DTINIVALITM  AND
      ITEMCOLE.DTFINVALITM <= @V_DTFINVALITM  AND

      ITEMCOLE.NRCOLEPRECO <> @P_NRCOLEPRECO AND

      ITEMCOLE.CDFILIAL   = @P_CDFILIAL  AND
      ITEMCOLE.CDPRODUTO  = @P_CDPRODUTO AND
      ITEMCOLE.CDREGIAO   = @P_CDREGIAO  AND
      ITEMCOLE.CDFORNECED = FORNECEDOR.CDFORNECED AND
      FORNECEDOR.IDTIPOFORN = 'N' AND
      ISNULL(ITEMCOLE.VRPRECECOITM * PARASUPRI.IDTIPOAPUR, ITEMCOLE.VRPRECUNIITM) =
          (SELECT ISNULL(MIN(ITEMCOLE.VRPRECECOITM * PARASUPRI.IDTIPOAPUR), MIN(ITEMCOLE.VRPRECUNIITM))
           FROM
              ITEMCOLE, FORNECEDOR, PARASUPRI
           WHERE
                PARASUPRI.CDFILIAL  = @P_CDFILIAL      AND
                ITEMCOLE.CDFILIAL   = @P_CDFILIAL      AND
                ITEMCOLE.CDPRODUTO  = @P_CDPRODUTO     AND
                ITEMCOLE.CDREGIAO   = @P_CDREGIAO      AND
                ITEMCOLE.NRCOLEPRECO <> @P_NRCOLEPRECO AND
                ITEMCOLE.CDFORNECED = FORNECEDOR.CDFORNECED AND
                FORNECEDOR.IDTIPOFORN = 'N')

   /* Menor preo econmico entre os fornecedores alternativos */
   SELECT
     @P_NMFORN_ALTER  = FORNECEDOR.NMFANTFORN,
     @P_VRFALTER_PRUN = ITEMCOLE.VRPRECUNIITM,
     @P_VRFALTER_ECO  = ITEMCOLE.VRPRECECOITM

   FROM FORNECEDOR,
        ITEMCOLE,
        PARASUPRI
   WHERE
      PARASUPRI.CDFILIAL  = @P_CDFILIAL   AND

      ITEMCOLE.DTINIVALITM >= @V_DTINIVALITM  AND
      ITEMCOLE.DTFINVALITM <= @V_DTFINVALITM  AND

      ITEMCOLE.NRCOLEPRECO <> @P_NRCOLEPRECO AND

      ITEMCOLE.CDFILIAL   = @P_CDFILIAL  AND
      ITEMCOLE.CDPRODUTO  = @P_CDPRODUTO AND
      ITEMCOLE.CDREGIAO   = @P_CDREGIAO  AND
      ITEMCOLE.CDFORNECED = FORNECEDOR.CDFORNECED AND
      FORNECEDOR.IDTIPOFORN = 'A' AND
      ISNULL(ITEMCOLE.VRPRECECOITM * PARASUPRI.IDTIPOAPUR, ITEMCOLE.VRPRECUNIITM) =
          (SELECT ISNULL(MIN(ITEMCOLE.VRPRECECOITM * PARASUPRI.IDTIPOAPUR), MIN(ITEMCOLE.VRPRECUNIITM))
           FROM
              ITEMCOLE, FORNECEDOR, PARASUPRI
           WHERE
                PARASUPRI.CDFILIAL  = @P_CDFILIAL      AND
                ITEMCOLE.CDFILIAL   = @P_CDFILIAL      AND
                ITEMCOLE.CDPRODUTO  = @P_CDPRODUTO     AND
                ITEMCOLE.CDREGIAO   = @P_CDREGIAO      AND
                ITEMCOLE.NRCOLEPRECO <> @P_NRCOLEPRECO AND
                ITEMCOLE.CDFORNECED = FORNECEDOR.CDFORNECED AND
                FORNECEDOR.IDTIPOFORN = 'A')

   SET @P_PERVARIACAO = 0;                
   IF @P_VRULTCOT_PRUN IS NOT NULL
   BEGIN
     IF NOT(@P_VRULTCOT_PRUN = 0) BEGIN
        SET @P_PERVARIACAO = ((@V_ATUAL_PR - @P_VRULTCOT_PRUN)/@P_VRULTCOT_PRUN)*100
     END
     ELSE BEGIN
        SET @P_PERVARIACAO = (@V_ATUAL_PR)*100
     END;
   END;
   SET @P_DTINIVAL_MINPRFN = @V_DTINIVALITM
   SET @P_DTFINVAL_MINPRFN = @V_DTFINVALITM
   SET @P_DTINIVAL_MINPRFA = @V_DTINIVALITM
   SET @P_DTFINVAL_MINPRFA = @V_DTFINVALITM
END;
GO
